// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https ://mozilla.org/MPL/2.0/.

#include "Angle.h"
#include <math.h>
#include "FloatSingle.h"

const float Rad2Deg = 57.29578F;
const float Deg2Rad = 0.0174532924F;
/*
float Angle::Normalize(float angle) {
  if (!isfinite(angle))
    return angle;

  while (angle <= -180)
    angle += 360;
  while (angle > 180)
    angle -= 360;
  return angle;
}

float Angle::Clamp(float angle, float min, float max) {
  float normalizedAngle = Normalize(angle);
  float r = Float::Clamp(normalizedAngle, min, max);
  return r;
}

float Angle::Difference(float a, float b) {
  float r = Normalize(b - a);
  return r;
}

float Angle::MoveTowards(float fromAngle, float toAngle, float maxAngle) {
  float d = toAngle - fromAngle;
  float sign = signbit(d) ? -1 : 1;
  d = sign * Float::Clamp(fabs(d), 0, maxAngle);
  return fromAngle + d;
}

float Angle::CosineRuleSide(float a, float b, float gamma) {
  float a2 = a * a;
  float b2 = b * b;
  float d = a2 + b2 - 2 * a * b * cos(gamma * Angle::Deg2Rad);
  // Catch edge cases where float inacuracies lead tot nans
  if (d < 0)
    return 0;

  float c = sqrtf(d);
  return c;
}

float Angle::CosineRuleAngle(float a, float b, float c) {
  float a2 = a * a;
  float b2 = b * b;
  float c2 = c * c;
  float d = (a2 + b2 - c2) / (2 * a * b);
  // Catch edge cases where float inacuracies lead tot nans
  if (d >= 1)
    return 0;
  if (d <= -1)
    return 180;

  float gamma = acos(d) * Angle::Rad2Deg;
  return gamma;
}

float Angle::SineRuleAngle(float a, float beta, float b) {
  float alpha = asin(a * sin(beta * Angle::Deg2Rad) / b);
  return alpha;
}
*/
//----------------------

template <>
AngleOf<float>::AngleOf(int angle) : value((float)angle) {}

template <>
AngleOf<float>::AngleOf(float angle) : value(angle) {}

// template <>
// AngleOf<float>::operator float() const {
//   return value;
// }

template <>
float AngleOf<float>::ToFloat() const {
  return value;
}

// template <>
// AngleOf<float> AngleOf<float>::pi = 3.1415927410125732421875F;

// template <>
// AngleOf<float> AngleOf<float>::Rad2Deg = 360.0f / (pi * 2);
// template <>
// AngleOf<float> AngleOf<float>::Deg2Rad = (pi * 2) / 360.0f;

template <>
bool AngleOf<float>::operator==(AngleOf<float> a) {
  return this->ToFloat() == a.ToFloat();
}

template <>
AngleOf<float> AngleOf<float>::operator-() const {
  AngleOf<float> angle = AngleOf(-this->value);
  return angle;
}

template <>
AngleOf<float> AngleOf<float>::operator-(const AngleOf<float>& a) const {
  AngleOf<float> angle = AngleOf();
  angle.value = this->value - a.value;
  return angle;
}

template <>
AngleOf<float> AngleOf<float>::operator+(const AngleOf<float>& a) const {
  AngleOf<float> angle = AngleOf();
  angle.value = this->value + a.value;
  return angle;
}

template <typename T>
AngleOf<T> AngleOf<T>::operator+=(const AngleOf<T>& a) {
  this->value += a.value;
  return *this;
}

template <typename T>
AngleOf<T> AngleOf<T>::Normalize(AngleOf<T> angle) {
  float angleValue = angle.ToFloat();
  if (!isfinite(angleValue))
    return angle;

  while (angleValue <= -180)
    angleValue += 360;
  while (angleValue > 180)
    angleValue -= 360;
  return AngleOf(angleValue);
}

template <>
AngleOf<float> AngleOf<float>::Clamp(AngleOf<float> angle,
                                     AngleOf<float> min,
                                     AngleOf<float> max) {
  float normalizedAngle = Normalize(angle).ToFloat();
  float r = Float::Clamp(normalizedAngle, min.ToFloat(), max.ToFloat());
  return r;
}

// template <typename T>
// Angle2<T> Angle2<T>::Difference(Angle2<T> a, Angle2<T> b) {
//   Angle2<T> r = Normalize(b - a);
//   return r;
// }

template <>
AngleOf<float> AngleOf<float>::MoveTowards(AngleOf<float> fromAngle,
                                           AngleOf<float> toAngle,
                                           AngleOf<float> maxAngle) {
  float d = toAngle.ToFloat() - fromAngle.ToFloat();
  int sign = signbit(d) ? -1 : 1;
  d = sign * Float::Clamp(fabsf(d), 0, maxAngle.ToFloat());
  return fromAngle.ToFloat() + d;
}

template <>
AngleOf<float> AngleOf<float>::CosineRuleSide(float a,
                                              float b,
                                              AngleOf<float> gamma) {
  float a2 = a * a;
  float b2 = b * b;
  float d = a2 + b2 -
            2 * a * b * cosf(gamma.ToFloat() * Passer::LinearAlgebra::Deg2Rad);
  // Catch edge cases where float inacuracies lead tot nans
  if (d < 0)
    return 0.0f;

  float c = sqrtf(d);
  return c;
}

template <>
AngleOf<float> AngleOf<float>::CosineRuleAngle(float a, float b, float c) {
  float a2 = a * a;
  float b2 = b * b;
  float c2 = c * c;
  float d = (a2 + b2 - c2) / (2 * a * b);
  // Catch edge cases where float inacuracies lead tot nans
  if (d >= 1)
    return 0.0f;
  if (d <= -1)
    return 180.0f;

  float gamma = acosf(d) * Rad2Deg;
  return gamma;
}

template <>
AngleOf<float> AngleOf<float>::SineRuleAngle(float a,
                                             AngleOf<float> beta,
                                             float b) {
  float deg2rad = Deg2Rad;
  float alpha = asinf(a * sinf(beta.ToFloat() * deg2rad) / b);
  return alpha;
}

template class AngleOf<float>;
template class AngleOf<signed short>;
